from forge import protos
from forge.config import config

stub = protos.WalletRpcStub(config.get_grpc_channel())

__wallet_type = protos.WalletType(pk=0, hash=1, address=1)


def create_wallet(
        wallet_type=__wallet_type, moniker='',
        passphrase='', req=None,
):
    """

    Parameters
    ----------
    req: RequestCreateWallet
    wallet_type: WalletType
    moniker: string
    passphrase: string

    Returns
    -------
    ResponseCreateWallet

    """
    if req is not None:
        return stub.create_wallet(req)
    else:
        req_kwargs = {
            'type': wallet_type,
            'moniker': moniker,
            'passphrase': passphrase,
        }
        return stub.create_wallet(
            protos.RequestCreateWallet(**req_kwargs),
        )


def load_wallet(address='', passphrase='', req=None):
    """
    rpc call to load wallet.

    parameters
    ----------
    req: requestloadwallet
    address: string
    passphrase: string

    returns
    -------
    responseloadwallet

    """
    if req is not None:
        return stub.load_wallet(req)
    else:
        req_kwargs = {
            'address': address,
            'passphrase': passphrase,
        }
        return stub.load_wallet(
            protos.RequestLoadWallet(**req_kwargs),
        )


def recover_wallet(
        passphrase='', moniker='', data=b'',
        wallet_type=__wallet_type, req=None,
):
    """
    rpc call to recover wallet with given passphrase.

    parameters
    ----------
    passphrase: string
    moniker: string
    req: requestrecoverwallet
    data: bytes
        data could be bytes of seed words or secret key
    wallet_type: walletType

    Returns
    -------
    ResponseRecoverWallet

    """
    if req is not None:
        return stub.recover_wallet(req)
    else:
        req_kwargs = {
            'data': data,
            'type': wallet_type,
            'passphrase': passphrase,
            'moniker': moniker,
        }
        req = protos.RequestRecoverWallet(**req_kwargs)
        print(req)
        return stub.recover_wallet(req)


def list_wallet():
    """
    RPC call to list wallets

    Returns
    -------
    stream ResponseListWallets

    """
    return stub.list_wallet(protos.RequestListWallet())


def remove_wallet(address='', req=None):
    """
    RPC call to remove wallet with given address

    Parameters
    ----------
    req: RequestRemoveWallet
    address: string

    Returns
    -------
    ResponseRemoveWallet

    """
    if req is not None:
        return stub.remove_wallet(req)
    else:
        req_kwargs = {
            'address': address,
        }
        return stub.remove_wallet(
            protos.RequestRemoveWallet(**req_kwargs),
        )


def declare_node(req):
    """

    Parameters
    ----------
    req: RequestDeclareNode

    Returns
    -------
    ResponseDeclareNode

    """
    if req is not None:
        return stub.declare_node(req)
    else:
        return stub.declare_node(protos.RequestDeclareNode())



import logging
import os
import sqlite3

from event_chain.config import config
from event_chain.db.utils import init_db

logger = logging.getLogger('ec-db')


def initialized_app_folder():
    app_path = config.app_path
    if not os.path.exists(app_path):
        os.system('mkdir -p {}'.format(app_path))
        logger.info("{} created.".format(app_path))
    else:
        logger.info("{} already exists.".format(app_path))


def reset_db():
    logger.info('Connecting to db {}'.format(config.db_path))
    conn = sqlite3.connect(config.db_path)
    if not conn:
        logger.error("Fail to connect to db".format(config.db_path))
    init_db(conn)
    logger.info('db has been initialized.'.format(config.db_path))
    return conn


if __name__ == '__main__':
    initialized_app_folder()
    reset_db()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: trace_type.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import type_pb2 as type__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='trace_type.proto',
  package='forge_abi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10trace_type.proto\x12\tforge_abi\x1a\ntype.proto\"(\n\tPageOrder\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"N\n\tPageInput\x12\x0e\n\x06\x63ursor\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\r\x12#\n\x05order\x18\x03 \x03(\x0b\x32\x14.forge_abi.PageOrder\"\x1b\n\nTypeFilter\x12\r\n\x05types\x18\x01 \x03(\t\"<\n\nTimeFilter\x12\x17\n\x0fstart_date_time\x18\x01 \x01(\t\x12\x15\n\rend_date_time\x18\x02 \x01(\t\"Z\n\rAddressFilter\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x10\n\x08receiver\x18\x02 \x01(\t\x12\'\n\tdirection\x18\x03 \x01(\x0e\x32\x14.forge_abi.Direction\"7\n\x08PageInfo\x12\x0e\n\x06\x63ursor\x18\x01 \x01(\t\x12\x0c\n\x04next\x18\x02 \x01(\x08\x12\r\n\x05total\x18\x03 \x01(\r\"\x94\x03\n\x12IndexedTransaction\x12\x0c\n\x04hash\x18\x01 \x01(\t\x12\x0e\n\x06sender\x18\x02 \x01(\t\x12\x10\n\x08receiver\x18\x03 \x01(\t\x12\x0c\n\x04time\x18\x04 \x01(\t\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\"\n\x02tx\x18\x06 \x01(\x0b\x32\x16.forge_abi.Transaction\x12\x37\n\rconsume_asset\x18\x07 \x01(\x0b\x32\x1e.forge_abi.IndexedConsumeAssetH\x00\x12\x35\n\x0c\x63reate_asset\x18\x08 \x01(\x0b\x32\x1d.forge_abi.IndexedCreateAssetH\x00\x12.\n\x08\x65xchange\x18\t \x01(\x0b\x32\x1a.forge_abi.IndexedExchangeH\x00\x12.\n\x08transfer\x18\n \x01(\x0b\x32\x1a.forge_abi.IndexedTransferH\x00\x12\x35\n\x0cupdate_asset\x18\x0b \x01(\x0b\x32\x1d.forge_abi.IndexedUpdateAssetH\x00\x42\x07\n\x05value\"\x8d\x03\n\x13IndexedAccountState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12#\n\x07\x62\x61lance\x18\x02 \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x12\n\nnum_assets\x18\x03 \x01(\x04\x12\x0f\n\x07num_txs\x18\x04 \x01(\x04\x12\r\n\x05nonce\x18\x05 \x01(\x04\x12\x14\n\x0cgenesis_time\x18\x06 \x01(\t\x12\x18\n\x10renaissance_time\x18\x07 \x01(\t\x12\x0f\n\x07moniker\x18\x08 \x01(\t\x12\x15\n\rmigrated_from\x18\t \x01(\t\x12\x13\n\x0bmigrated_to\x18\n \x01(\t\x12\x31\n\x15total_received_stakes\x18\x0b \x01(\x0b\x32\x12.forge_abi.BigUint\x12(\n\x0ctotal_stakes\x18\x0c \x01(\x0b\x32\x12.forge_abi.BigUint\x12*\n\x0etotal_unstakes\x18\r \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x16\n\x0erecent_num_txs\x18\x0e \x03(\x04\"\x86\x01\n\x11IndexedAssetState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\r\n\x05owner\x18\x02 \x01(\t\x12\x14\n\x0cgenesis_time\x18\x03 \x01(\t\x12\x18\n\x10renaissance_time\x18\x04 \x01(\t\x12\x0f\n\x07moniker\x18\x05 \x01(\t\x12\x10\n\x08readonly\x18\x06 \x01(\x08\"\xba\x01\n\x11IndexedStakeState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12#\n\x07\x62\x61lance\x18\x02 \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x10\n\x08receiver\x18\x04 \x01(\t\x12\x14\n\x0cgenesis_time\x18\x05 \x01(\t\x12\x18\n\x10renaissance_time\x18\x06 \x01(\t\x12\x0f\n\x07message\x18\x07 \x01(\t\x12\x0c\n\x04type\x18\x08 \x01(\r\"$\n\x13IndexedConsumeAsset\x12\r\n\x05\x61sset\x18\x01 \x01(\t\"#\n\x12IndexedCreateAsset\x12\r\n\x05\x61sset\x18\x01 \x01(\t\"A\n\x0fIndexedExchange\x12\x15\n\rsender_assets\x18\x01 \x03(\t\x12\x17\n\x0freceiver_assets\x18\x02 \x03(\t\"!\n\x0fIndexedTransfer\x12\x0e\n\x06\x61ssets\x18\x01 \x03(\t\"#\n\x12IndexedUpdateAsset\x12\r\n\x05\x61sset\x18\x01 \x01(\t\"6\n\x0cHeightFilter\x12\x13\n\x0b\x66rom_height\x18\x01 \x01(\x04\x12\x11\n\tto_height\x18\x02 \x01(\x04\"8\n\x0cNumTxsFilter\x12\x13\n\x0bmin_num_txs\x18\x01 \x01(\x04\x12\x13\n\x0bmax_num_txs\x18\x02 \x01(\x04\"O\n\x13NumInvalidTxsFilter\x12\x1b\n\x13min_num_invalid_txs\x18\x01 \x01(\x04\x12\x1b\n\x13max_num_invalid_txs\x18\x02 \x01(\x04\"h\n\x0cIndexedBlock\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0c\n\x04time\x18\x02 \x01(\t\x12\x10\n\x08proposer\x18\x03 \x01(\t\x12\x0f\n\x07num_txs\x18\x04 \x01(\x04\x12\x17\n\x0fnum_invalid_txs\x18\x05 \x01(\x04\"\xba\x01\n\x0cHealthStatus\x12-\n\tconsensus\x18\x01 \x01(\x0b\x32\x1a.forge_abi.ConsensusStatus\x12)\n\x07network\x18\x02 \x01(\x0b\x32\x18.forge_abi.NetworkStatus\x12)\n\x07storage\x18\x03 \x01(\x0b\x32\x18.forge_abi.StorageStatus\x12%\n\x05\x66orge\x18\x04 \x01(\x0b\x32\x16.forge_abi.ForgeStatus\"G\n\x0f\x43onsensusStatus\x12\x0e\n\x06health\x18\x01 \x01(\x08\x12\x0e\n\x06synced\x18\x02 \x01(\x08\x12\x14\n\x0c\x62lock_height\x18\x03 \x01(\x04\"2\n\rNetworkStatus\x12\x0e\n\x06health\x18\x01 \x01(\x08\x12\x11\n\tnum_peers\x18\x02 \x01(\r\"y\n\rStorageStatus\x12\x0e\n\x06health\x18\x01 \x01(\x08\x12\x16\n\x0eindexer_server\x18\x02 \x01(\t\x12\x10\n\x08state_db\x18\x03 \x01(\t\x12.\n\ndisk_space\x18\x04 \x01(\x0b\x32\x1a.forge_abi.DiskSpaceStatus\"5\n\x0f\x44iskSpaceStatus\x12\x13\n\x0b\x66orge_usage\x18\x01 \x01(\t\x12\r\n\x05total\x18\x02 \x01(\t\"v\n\x0b\x46orgeStatus\x12\x0e\n\x06health\x18\x01 \x01(\x08\x12\x12\n\nabi_server\x18\x02 \x01(\t\x12\x11\n\tforge_web\x18\x03 \x01(\t\x12\x30\n\x0b\x61\x62\x63i_server\x18\x04 \x01(\x0b\x32\x1b.forge_abi.AbciServerStatus\"=\n\x10\x41\x62\x63iServerStatus\x12\x16\n\x0e\x61\x62\x63i_consensus\x18\x01 \x01(\t\x12\x11\n\tabci_info\x18\x02 \x01(\t*/\n\tDirection\x12\n\n\x06mutual\x10\x00\x12\x0b\n\x07one_way\x10\x01\x12\t\n\x05union\x10\x02\x62\x06proto3')
  ,
  dependencies=[type__pb2.DESCRIPTOR,])

_DIRECTION = _descriptor.EnumDescriptor(
  name='Direction',
  full_name='forge_abi.Direction',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='mutual', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='one_way', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='union', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2728,
  serialized_end=2775,
)
_sym_db.RegisterEnumDescriptor(_DIRECTION)

Direction = enum_type_wrapper.EnumTypeWrapper(_DIRECTION)
mutual = 0
one_way = 1
union = 2



_PAGEORDER = _descriptor.Descriptor(
  name='PageOrder',
  full_name='forge_abi.PageOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='forge_abi.PageOrder.field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.PageOrder.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=83,
)


_PAGEINPUT = _descriptor.Descriptor(
  name='PageInput',
  full_name='forge_abi.PageInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cursor', full_name='forge_abi.PageInput.cursor', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='forge_abi.PageInput.size', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order', full_name='forge_abi.PageInput.order', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=163,
)


_TYPEFILTER = _descriptor.Descriptor(
  name='TypeFilter',
  full_name='forge_abi.TypeFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='types', full_name='forge_abi.TypeFilter.types', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=165,
  serialized_end=192,
)


_TIMEFILTER = _descriptor.Descriptor(
  name='TimeFilter',
  full_name='forge_abi.TimeFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_date_time', full_name='forge_abi.TimeFilter.start_date_time', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_date_time', full_name='forge_abi.TimeFilter.end_date_time', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=254,
)


_ADDRESSFILTER = _descriptor.Descriptor(
  name='AddressFilter',
  full_name='forge_abi.AddressFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.AddressFilter.sender', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.AddressFilter.receiver', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='direction', full_name='forge_abi.AddressFilter.direction', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=256,
  serialized_end=346,
)


_PAGEINFO = _descriptor.Descriptor(
  name='PageInfo',
  full_name='forge_abi.PageInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cursor', full_name='forge_abi.PageInfo.cursor', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next', full_name='forge_abi.PageInfo.next', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total', full_name='forge_abi.PageInfo.total', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=348,
  serialized_end=403,
)


_INDEXEDTRANSACTION = _descriptor.Descriptor(
  name='IndexedTransaction',
  full_name='forge_abi.IndexedTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='forge_abi.IndexedTransaction.hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.IndexedTransaction.sender', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.IndexedTransaction.receiver', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='forge_abi.IndexedTransaction.time', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.IndexedTransaction.type', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx', full_name='forge_abi.IndexedTransaction.tx', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='consume_asset', full_name='forge_abi.IndexedTransaction.consume_asset', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='create_asset', full_name='forge_abi.IndexedTransaction.create_asset', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='forge_abi.IndexedTransaction.exchange', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transfer', full_name='forge_abi.IndexedTransaction.transfer', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_asset', full_name='forge_abi.IndexedTransaction.update_asset', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='forge_abi.IndexedTransaction.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=406,
  serialized_end=810,
)


_INDEXEDACCOUNTSTATE = _descriptor.Descriptor(
  name='IndexedAccountState',
  full_name='forge_abi.IndexedAccountState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedAccountState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='balance', full_name='forge_abi.IndexedAccountState.balance', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_assets', full_name='forge_abi.IndexedAccountState.num_assets', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_txs', full_name='forge_abi.IndexedAccountState.num_txs', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='forge_abi.IndexedAccountState.nonce', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedAccountState.genesis_time', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedAccountState.renaissance_time', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='moniker', full_name='forge_abi.IndexedAccountState.moniker', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='migrated_from', full_name='forge_abi.IndexedAccountState.migrated_from', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='migrated_to', full_name='forge_abi.IndexedAccountState.migrated_to', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_received_stakes', full_name='forge_abi.IndexedAccountState.total_received_stakes', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_stakes', full_name='forge_abi.IndexedAccountState.total_stakes', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_unstakes', full_name='forge_abi.IndexedAccountState.total_unstakes', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recent_num_txs', full_name='forge_abi.IndexedAccountState.recent_num_txs', index=13,
      number=14, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=813,
  serialized_end=1210,
)


_INDEXEDASSETSTATE = _descriptor.Descriptor(
  name='IndexedAssetState',
  full_name='forge_abi.IndexedAssetState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedAssetState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner', full_name='forge_abi.IndexedAssetState.owner', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedAssetState.genesis_time', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedAssetState.renaissance_time', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='moniker', full_name='forge_abi.IndexedAssetState.moniker', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly', full_name='forge_abi.IndexedAssetState.readonly', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1213,
  serialized_end=1347,
)


_INDEXEDSTAKESTATE = _descriptor.Descriptor(
  name='IndexedStakeState',
  full_name='forge_abi.IndexedStakeState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedStakeState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='balance', full_name='forge_abi.IndexedStakeState.balance', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.IndexedStakeState.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.IndexedStakeState.receiver', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedStakeState.genesis_time', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedStakeState.renaissance_time', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='forge_abi.IndexedStakeState.message', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.IndexedStakeState.type', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1350,
  serialized_end=1536,
)


_INDEXEDCONSUMEASSET = _descriptor.Descriptor(
  name='IndexedConsumeAsset',
  full_name='forge_abi.IndexedConsumeAsset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='asset', full_name='forge_abi.IndexedConsumeAsset.asset', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1538,
  serialized_end=1574,
)


_INDEXEDCREATEASSET = _descriptor.Descriptor(
  name='IndexedCreateAsset',
  full_name='forge_abi.IndexedCreateAsset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='asset', full_name='forge_abi.IndexedCreateAsset.asset', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1576,
  serialized_end=1611,
)


_INDEXEDEXCHANGE = _descriptor.Descriptor(
  name='IndexedExchange',
  full_name='forge_abi.IndexedExchange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender_assets', full_name='forge_abi.IndexedExchange.sender_assets', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver_assets', full_name='forge_abi.IndexedExchange.receiver_assets', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1613,
  serialized_end=1678,
)


_INDEXEDTRANSFER = _descriptor.Descriptor(
  name='IndexedTransfer',
  full_name='forge_abi.IndexedTransfer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='assets', full_name='forge_abi.IndexedTransfer.assets', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1680,
  serialized_end=1713,
)


_INDEXEDUPDATEASSET = _descriptor.Descriptor(
  name='IndexedUpdateAsset',
  full_name='forge_abi.IndexedUpdateAsset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='asset', full_name='forge_abi.IndexedUpdateAsset.asset', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1715,
  serialized_end=1750,
)


_HEIGHTFILTER = _descriptor.Descriptor(
  name='HeightFilter',
  full_name='forge_abi.HeightFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='from_height', full_name='forge_abi.HeightFilter.from_height', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='to_height', full_name='forge_abi.HeightFilter.to_height', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1752,
  serialized_end=1806,
)


_NUMTXSFILTER = _descriptor.Descriptor(
  name='NumTxsFilter',
  full_name='forge_abi.NumTxsFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_num_txs', full_name='forge_abi.NumTxsFilter.min_num_txs', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_num_txs', full_name='forge_abi.NumTxsFilter.max_num_txs', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1808,
  serialized_end=1864,
)


_NUMINVALIDTXSFILTER = _descriptor.Descriptor(
  name='NumInvalidTxsFilter',
  full_name='forge_abi.NumInvalidTxsFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_num_invalid_txs', full_name='forge_abi.NumInvalidTxsFilter.min_num_invalid_txs', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_num_invalid_txs', full_name='forge_abi.NumInvalidTxsFilter.max_num_invalid_txs', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1866,
  serialized_end=1945,
)


_INDEXEDBLOCK = _descriptor.Descriptor(
  name='IndexedBlock',
  full_name='forge_abi.IndexedBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='forge_abi.IndexedBlock.height', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='forge_abi.IndexedBlock.time', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proposer', full_name='forge_abi.IndexedBlock.proposer', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_txs', full_name='forge_abi.IndexedBlock.num_txs', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_invalid_txs', full_name='forge_abi.IndexedBlock.num_invalid_txs', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1947,
  serialized_end=2051,
)


_HEALTHSTATUS = _descriptor.Descriptor(
  name='HealthStatus',
  full_name='forge_abi.HealthStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='consensus', full_name='forge_abi.HealthStatus.consensus', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='network', full_name='forge_abi.HealthStatus.network', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storage', full_name='forge_abi.HealthStatus.storage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='forge', full_name='forge_abi.HealthStatus.forge', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2054,
  serialized_end=2240,
)


_CONSENSUSSTATUS = _descriptor.Descriptor(
  name='ConsensusStatus',
  full_name='forge_abi.ConsensusStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='health', full_name='forge_abi.ConsensusStatus.health', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='synced', full_name='forge_abi.ConsensusStatus.synced', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_height', full_name='forge_abi.ConsensusStatus.block_height', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2242,
  serialized_end=2313,
)


_NETWORKSTATUS = _descriptor.Descriptor(
  name='NetworkStatus',
  full_name='forge_abi.NetworkStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='health', full_name='forge_abi.NetworkStatus.health', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_peers', full_name='forge_abi.NetworkStatus.num_peers', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2315,
  serialized_end=2365,
)


_STORAGESTATUS = _descriptor.Descriptor(
  name='StorageStatus',
  full_name='forge_abi.StorageStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='health', full_name='forge_abi.StorageStatus.health', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indexer_server', full_name='forge_abi.StorageStatus.indexer_server', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state_db', full_name='forge_abi.StorageStatus.state_db', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='disk_space', full_name='forge_abi.StorageStatus.disk_space', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2367,
  serialized_end=2488,
)


_DISKSPACESTATUS = _descriptor.Descriptor(
  name='DiskSpaceStatus',
  full_name='forge_abi.DiskSpaceStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='forge_usage', full_name='forge_abi.DiskSpaceStatus.forge_usage', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total', full_name='forge_abi.DiskSpaceStatus.total', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2490,
  serialized_end=2543,
)


_FORGESTATUS = _descriptor.Descriptor(
  name='ForgeStatus',
  full_name='forge_abi.ForgeStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='health', full_name='forge_abi.ForgeStatus.health', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='abi_server', full_name='forge_abi.ForgeStatus.abi_server', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='forge_web', full_name='forge_abi.ForgeStatus.forge_web', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='abci_server', full_name='forge_abi.ForgeStatus.abci_server', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2545,
  serialized_end=2663,
)


_ABCISERVERSTATUS = _descriptor.Descriptor(
  name='AbciServerStatus',
  full_name='forge_abi.AbciServerStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='abci_consensus', full_name='forge_abi.AbciServerStatus.abci_consensus', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='abci_info', full_name='forge_abi.AbciServerStatus.abci_info', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2665,
  serialized_end=2726,
)

_PAGEINPUT.fields_by_name['order'].message_type = _PAGEORDER
_ADDRESSFILTER.fields_by_name['direction'].enum_type = _DIRECTION
_INDEXEDTRANSACTION.fields_by_name['tx'].message_type = type__pb2._TRANSACTION
_INDEXEDTRANSACTION.fields_by_name['consume_asset'].message_type = _INDEXEDCONSUMEASSET
_INDEXEDTRANSACTION.fields_by_name['create_asset'].message_type = _INDEXEDCREATEASSET
_INDEXEDTRANSACTION.fields_by_name['exchange'].message_type = _INDEXEDEXCHANGE
_INDEXEDTRANSACTION.fields_by_name['transfer'].message_type = _INDEXEDTRANSFER
_INDEXEDTRANSACTION.fields_by_name['update_asset'].message_type = _INDEXEDUPDATEASSET
_INDEXEDTRANSACTION.oneofs_by_name['value'].fields.append(
  _INDEXEDTRANSACTION.fields_by_name['consume_asset'])
_INDEXEDTRANSACTION.fields_by_name['consume_asset'].containing_oneof = _INDEXEDTRANSACTION.oneofs_by_name['value']
_INDEXEDTRANSACTION.oneofs_by_name['value'].fields.append(
  _INDEXEDTRANSACTION.fields_by_name['create_asset'])
_INDEXEDTRANSACTION.fields_by_name['create_asset'].containing_oneof = _INDEXEDTRANSACTION.oneofs_by_name['value']
_INDEXEDTRANSACTION.oneofs_by_name['value'].fields.append(
  _INDEXEDTRANSACTION.fields_by_name['exchange'])
_INDEXEDTRANSACTION.fields_by_name['exchange'].containing_oneof = _INDEXEDTRANSACTION.oneofs_by_name['value']
_INDEXEDTRANSACTION.oneofs_by_name['value'].fields.append(
  _INDEXEDTRANSACTION.fields_by_name['transfer'])
_INDEXEDTRANSACTION.fields_by_name['transfer'].containing_oneof = _INDEXEDTRANSACTION.oneofs_by_name['value']
_INDEXEDTRANSACTION.oneofs_by_name['value'].fields.append(
  _INDEXEDTRANSACTION.fields_by_name['update_asset'])
_INDEXEDTRANSACTION.fields_by_name['update_asset'].containing_oneof = _INDEXEDTRANSACTION.oneofs_by_name['value']
_INDEXEDACCOUNTSTATE.fields_by_name['balance'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_received_stakes'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_stakes'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_unstakes'].message_type = type__pb2._BIGUINT
_INDEXEDSTAKESTATE.fields_by_name['balance'].message_type = type__pb2._BIGUINT
_HEALTHSTATUS.fields_by_name['consensus'].message_type = _CONSENSUSSTATUS
_HEALTHSTATUS.fields_by_name['network'].message_type = _NETWORKSTATUS
_HEALTHSTATUS.fields_by_name['storage'].message_type = _STORAGESTATUS
_HEALTHSTATUS.fields_by_name['forge'].message_type = _FORGESTATUS
_STORAGESTATUS.fields_by_name['disk_space'].message_type = _DISKSPACESTATUS
_FORGESTATUS.fields_by_name['abci_server'].message_type = _ABCISERVERSTATUS
DESCRIPTOR.message_types_by_name['PageOrder'] = _PAGEORDER
DESCRIPTOR.message_types_by_name['PageInput'] = _PAGEINPUT
DESCRIPTOR.message_types_by_name['TypeFilter'] = _TYPEFILTER
DESCRIPTOR.message_types_by_name['TimeFilter'] = _TIMEFILTER
DESCRIPTOR.message_types_by_name['AddressFilter'] = _ADDRESSFILTER
DESCRIPTOR.message_types_by_name['PageInfo'] = _PAGEINFO
DESCRIPTOR.message_types_by_name['IndexedTransaction'] = _INDEXEDTRANSACTION
DESCRIPTOR.message_types_by_name['IndexedAccountState'] = _INDEXEDACCOUNTSTATE
DESCRIPTOR.message_types_by_name['IndexedAssetState'] = _INDEXEDASSETSTATE
DESCRIPTOR.message_types_by_name['IndexedStakeState'] = _INDEXEDSTAKESTATE
DESCRIPTOR.message_types_by_name['IndexedConsumeAsset'] = _INDEXEDCONSUMEASSET
DESCRIPTOR.message_types_by_name['IndexedCreateAsset'] = _INDEXEDCREATEASSET
DESCRIPTOR.message_types_by_name['IndexedExchange'] = _INDEXEDEXCHANGE
DESCRIPTOR.message_types_by_name['IndexedTransfer'] = _INDEXEDTRANSFER
DESCRIPTOR.message_types_by_name['IndexedUpdateAsset'] = _INDEXEDUPDATEASSET
DESCRIPTOR.message_types_by_name['HeightFilter'] = _HEIGHTFILTER
DESCRIPTOR.message_types_by_name['NumTxsFilter'] = _NUMTXSFILTER
DESCRIPTOR.message_types_by_name['NumInvalidTxsFilter'] = _NUMINVALIDTXSFILTER
DESCRIPTOR.message_types_by_name['IndexedBlock'] = _INDEXEDBLOCK
DESCRIPTOR.message_types_by_name['HealthStatus'] = _HEALTHSTATUS
DESCRIPTOR.message_types_by_name['ConsensusStatus'] = _CONSENSUSSTATUS
DESCRIPTOR.message_types_by_name['NetworkStatus'] = _NETWORKSTATUS
DESCRIPTOR.message_types_by_name['StorageStatus'] = _STORAGESTATUS
DESCRIPTOR.message_types_by_name['DiskSpaceStatus'] = _DISKSPACESTATUS
DESCRIPTOR.message_types_by_name['ForgeStatus'] = _FORGESTATUS
DESCRIPTOR.message_types_by_name['AbciServerStatus'] = _ABCISERVERSTATUS
DESCRIPTOR.enum_types_by_name['Direction'] = _DIRECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PageOrder = _reflection.GeneratedProtocolMessageType('PageOrder', (_message.Message,), dict(
  DESCRIPTOR = _PAGEORDER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageOrder)
  ))
_sym_db.RegisterMessage(PageOrder)

PageInput = _reflection.GeneratedProtocolMessageType('PageInput', (_message.Message,), dict(
  DESCRIPTOR = _PAGEINPUT,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageInput)
  ))
_sym_db.RegisterMessage(PageInput)

TypeFilter = _reflection.GeneratedProtocolMessageType('TypeFilter', (_message.Message,), dict(
  DESCRIPTOR = _TYPEFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.TypeFilter)
  ))
_sym_db.RegisterMessage(TypeFilter)

TimeFilter = _reflection.GeneratedProtocolMessageType('TimeFilter', (_message.Message,), dict(
  DESCRIPTOR = _TIMEFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.TimeFilter)
  ))
_sym_db.RegisterMessage(TimeFilter)

AddressFilter = _reflection.GeneratedProtocolMessageType('AddressFilter', (_message.Message,), dict(
  DESCRIPTOR = _ADDRESSFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.AddressFilter)
  ))
_sym_db.RegisterMessage(AddressFilter)

PageInfo = _reflection.GeneratedProtocolMessageType('PageInfo', (_message.Message,), dict(
  DESCRIPTOR = _PAGEINFO,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageInfo)
  ))
_sym_db.RegisterMessage(PageInfo)

IndexedTransaction = _reflection.GeneratedProtocolMessageType('IndexedTransaction', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDTRANSACTION,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedTransaction)
  ))
_sym_db.RegisterMessage(IndexedTransaction)

IndexedAccountState = _reflection.GeneratedProtocolMessageType('IndexedAccountState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDACCOUNTSTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedAccountState)
  ))
_sym_db.RegisterMessage(IndexedAccountState)

IndexedAssetState = _reflection.GeneratedProtocolMessageType('IndexedAssetState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDASSETSTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedAssetState)
  ))
_sym_db.RegisterMessage(IndexedAssetState)

IndexedStakeState = _reflection.GeneratedProtocolMessageType('IndexedStakeState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDSTAKESTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedStakeState)
  ))
_sym_db.RegisterMessage(IndexedStakeState)

IndexedConsumeAsset = _reflection.GeneratedProtocolMessageType('IndexedConsumeAsset', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDCONSUMEASSET,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedConsumeAsset)
  ))
_sym_db.RegisterMessage(IndexedConsumeAsset)

IndexedCreateAsset = _reflection.GeneratedProtocolMessageType('IndexedCreateAsset', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDCREATEASSET,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedCreateAsset)
  ))
_sym_db.RegisterMessage(IndexedCreateAsset)

IndexedExchange = _reflection.GeneratedProtocolMessageType('IndexedExchange', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDEXCHANGE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedExchange)
  ))
_sym_db.RegisterMessage(IndexedExchange)

IndexedTransfer = _reflection.GeneratedProtocolMessageType('IndexedTransfer', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDTRANSFER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedTransfer)
  ))
_sym_db.RegisterMessage(IndexedTransfer)

IndexedUpdateAsset = _reflection.GeneratedProtocolMessageType('IndexedUpdateAsset', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDUPDATEASSET,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedUpdateAsset)
  ))
_sym_db.RegisterMessage(IndexedUpdateAsset)

HeightFilter = _reflection.GeneratedProtocolMessageType('HeightFilter', (_message.Message,), dict(
  DESCRIPTOR = _HEIGHTFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.HeightFilter)
  ))
_sym_db.RegisterMessage(HeightFilter)

NumTxsFilter = _reflection.GeneratedProtocolMessageType('NumTxsFilter', (_message.Message,), dict(
  DESCRIPTOR = _NUMTXSFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.NumTxsFilter)
  ))
_sym_db.RegisterMessage(NumTxsFilter)

NumInvalidTxsFilter = _reflection.GeneratedProtocolMessageType('NumInvalidTxsFilter', (_message.Message,), dict(
  DESCRIPTOR = _NUMINVALIDTXSFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.NumInvalidTxsFilter)
  ))
_sym_db.RegisterMessage(NumInvalidTxsFilter)

IndexedBlock = _reflection.GeneratedProtocolMessageType('IndexedBlock', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDBLOCK,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedBlock)
  ))
_sym_db.RegisterMessage(IndexedBlock)

HealthStatus = _reflection.GeneratedProtocolMessageType('HealthStatus', (_message.Message,), dict(
  DESCRIPTOR = _HEALTHSTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.HealthStatus)
  ))
_sym_db.RegisterMessage(HealthStatus)

ConsensusStatus = _reflection.GeneratedProtocolMessageType('ConsensusStatus', (_message.Message,), dict(
  DESCRIPTOR = _CONSENSUSSTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.ConsensusStatus)
  ))
_sym_db.RegisterMessage(ConsensusStatus)

NetworkStatus = _reflection.GeneratedProtocolMessageType('NetworkStatus', (_message.Message,), dict(
  DESCRIPTOR = _NETWORKSTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.NetworkStatus)
  ))
_sym_db.RegisterMessage(NetworkStatus)

StorageStatus = _reflection.GeneratedProtocolMessageType('StorageStatus', (_message.Message,), dict(
  DESCRIPTOR = _STORAGESTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.StorageStatus)
  ))
_sym_db.RegisterMessage(StorageStatus)

DiskSpaceStatus = _reflection.GeneratedProtocolMessageType('DiskSpaceStatus', (_message.Message,), dict(
  DESCRIPTOR = _DISKSPACESTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.DiskSpaceStatus)
  ))
_sym_db.RegisterMessage(DiskSpaceStatus)

ForgeStatus = _reflection.GeneratedProtocolMessageType('ForgeStatus', (_message.Message,), dict(
  DESCRIPTOR = _FORGESTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.ForgeStatus)
  ))
_sym_db.RegisterMessage(ForgeStatus)

AbciServerStatus = _reflection.GeneratedProtocolMessageType('AbciServerStatus', (_message.Message,), dict(
  DESCRIPTOR = _ABCISERVERSTATUS,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.AbciServerStatus)
  ))
_sym_db.RegisterMessage(AbciServerStatus)


# @@protoc_insertion_point(module_scope)



import base64
import json
import logging
from enum import Enum

import base58
import event_chain.protos as protos
from google.protobuf.any_pb2 import Any
from google.protobuf.timestamp_pb2 import Timestamp

from forge.utils import utils as forge_utils

logger = logging.getLogger('ec-helpers')


def gen_timestamp(datetime):
    res = Timestamp()
    res.FromDatetime(datetime)
    return res


def add_to_proto_list(info, repeated):
    res = {item for item in repeated}
    res.add(info)
    return res


def remove_from_proto_list(info, repeated):
    res = filter(lambda item: item != info, repeated)
    return res


class WalletResponse:
    def __init__(self, response):
        self.response = response
        self.user_info = response.get('userInfo')
        self.decoded_info = self.decode_user_info()
        self.requested_claim = self.decoded_info.get('requestedClaims')[0]

    def get_user_pk(self):
        pk = self.response.get('userPk')
        logger.debug("Got userpk from wallet {}".format(pk))
        return forge_utils.multibase_b58decode(pk)

    def decode_user_info(self):
        if not self.user_info:
            logger.error(
                "Fail to parse user_info from this Response {}.".format(
                    self.response,
                ),
            )
        else:
            sig = self.user_info.split('.')[1]
            decoded = base64.urlsafe_b64decode(
                (sig + '=' * (-len(sig) % 4)).encode(),
            ).decode()
            dict = json.loads(decoded)
            logger.debug("User info is decoded successfully. {}".format(dict))
            return dict

    def get_origin_tx(self):
        origin = self.requested_claim.get('origin')
        origin = str(origin)
        logger.debug(
            "Wallet Response:origin tx before decode: {}".format(origin),
        )
        decoded = base58.b58decode(origin[1:])
        tx = protos.Transaction()
        tx.ParseFromString(decoded)
        logger.debug(
            "Wallet Response:origin tx after base58 decode: {}".format(tx),
        )
        return tx

    def get_address(self):
        did = self.decoded_info.get('iss')
        logger.debug("Wallet Response:raw address: {}".format(did))
        did = str(did)
        return did.split(':')[-1]

    def get_signature(self):
        sig = self.requested_claim.get('sig')
        logger.debug("Wallet Response:raw sig {}: ".format(sig))
        str_sig = str(sig)
        decoded_sig = base58.b58decode(str_sig[1:])
        logger.debug(
            "Wallet Response:sig after base58 decode: {}".format(
                decoded_sig),
        )
        return decoded_sig

    def get_asset_address(self):
        asset_address = self.requested_claim.get('did')
        if not asset_address:
            return None
        else:
            asset_address = str(asset_address)
            logger.debug(
                "Wallet Response: asset_address: {}".format(asset_address),
            )
            return asset_address

    # def get_event_address(self):
    #     tx = self.requested_claim.get('tx')
    #     decoded_tx = base64.urlsafe_b64decode(
    #             (tx + '=' * (-len(tx) % 4)).encode('utf8'))
    #
    #     parsed_tx = protos.Transaction()
    #     parsed_tx.ParseFromString(decoded_tx)
    #     logger.debug("Parsed tx successfully! {}".format(parsed_tx))
    #     exchange_itx = forge_utils.parse_to_proto(
    #             parsed_tx.itx.value,
    #             protos.ExchangeTx,
    #     )
    #     event_address = exchange_itx.data.value.decode('utf8')
    #     return event_address


def add_multi_sig_to_tx(tx, address, signature, user_pk):
    logger.debug("Adding multisig to tx...")
    logger.debug("tx: {}".format(tx))
    logger.debug("address: {}".format(address))
    logger.debug("signature: {}".format(signature))
    # #
    # # parsed_address = tx.signatures[0].signer
    # # assert (address == parsed_address)
    # multisig = protos.Multisig(
    #     signer=address,
    #     signature=signature,
    # )
    # parmas = {
    #     'from': getattr(tx, 'from'),
    #     'nonce': tx.nonce,
    #     'signature': tx.signature,
    #     'chain_id': tx.chain_id,
    #     'signatures': [multisig],
    #     'itx': tx.itx,
    # }
    new_tx = update_tx_multisig(tx, address, user_pk, signature)
    logger.debug("Address and signature has been added to tx: ")
    logger.debug("new tx: {}".format(new_tx))

    return new_tx


def to_display_time(timestamp):
    dt = timestamp.ToDatetime()
    return dt.strftime("%a, %b %d, %Y")


def time_diff(t1, t2):
    return t2.ToDatetime() - t1.ToDatetime()


def update_tx_multisig(tx, signer, pk, signature=None, data=None):
    multisig = protos.Multisig(
        signer=signer,
        signature=signature,
        data=data,
        pk=pk,
    )
    params = {
        'from': getattr(tx, 'from'),
        'nonce': tx.nonce,
        'signature': tx.signature,
        'chain_id': tx.chain_id,
        'signatures': [multisig],
        'itx': tx.itx,
        'pk': tx.pk,
    }
    new_tx = protos.Transaction(**params)
    return new_tx


def encode_string_to_any(type_url, str):
    return Any(
        type_url=type_url,
        value=str.encode(),
    )


def base58_encode_tx(tx):
    b = tx.SerializeToString()
    return forge_utils.multibase_b58encode(b)


class ForgeTxType(Enum):
    ACTIVATE_ASSET = 'fg:t:activate_asset'
    CREATE_ASSET = 'fg:t:create_asset'
    EXCHANGE = 'fg:t:exchange'



from forge import protos
from forge.config import config
from forge.utils import utils

stub = protos.FileRpcStub(config.get_grpc_channel())


def store_file(chunk=b'', req=None):
    """
    RPC call to store file

    Parameters
    ----------
    req: stream RequestStoreFile
    chunk: iterator of bytes

    Returns
    -------
    ResponseStoreFile

    """

    def to_req(item):
        if isinstance(item, protos.RequestStoreFile):
            return item
        else:
            return protos.RequestStoreFile(chunk=item)

    if req is not None:
        return stub.store_file(utils.to_iter(to_req, req))
    else:
        chunks = utils.to_iter(to_req, chunk)
        return stub.store_file(chunks)


def load_file(file_hash='', req=None):
    """
    RPC call to load file.

    Parameters
    ----------
    req: RequestLoadFile
    file_hash: string

    Returns
    -------
    stream ResponseLoadFile

    """
    if req is not None:
        return stub.load_file(req)
    else:
        req_kwargs = {
            'hash': file_hash,
        }
        return stub.load_file(protos.RequestLoadFile(**req_kwargs))



import logging
import os
import os.path as path
from os.path import expanduser

import grpc
import toml
from deepmerge import Merger

logger = logging.getLogger('forge-config')

default_forge_toml = path.join(path.dirname(__file__), "forge_default.toml")

config = toml.load(default_forge_toml)


def use_config(user_config=None, env=True):
    # Use Environment FORGE_CONFIG if exists
    env_config = os.environ.get('FORGE_CONFIG')
    if env:
        logger.info("Reading config from environment...")
        merge_config(env_config)
    elif user_config:
        logger.info("FORGE_CONFIG not found!")
        merge_config(user_config)
    else:
        logger.error("Please either specify a config path or "
                     "set FORGE_CONFIG in the environment!")


def merge_config(config_path):
    if path.exists(config_path):
        logger.info("Using config in {}".format(config_path))
        try:
            user_config = toml.load(config_path)

            merger = Merger(
                    # strategy for each type
                    [
                        (list, ["append"]),
                        (dict, ["merge"]),
                    ],
                    # fallback strategies for all other types
                    ["override"],
                    # strategies for conflicted types
                    ["override"],
            )
            res_config = toml.load(default_forge_toml)
            merger.merge(res_config, user_config)
            global config
            config = res_config

        except Exception as e:
            logger.error(e, exc_info=True)
            logger.error("Fail to parse toml config in {}".format(config_path))


use_config()


def __parse_socket(forge_path, forge_socket):
    """

    Parameters
    ----------
    Returns: string
        unix: "unix:///tmp/.forge_test/app/socks/abi.sock"
        tcp: "tcp://127.0.0.1:38210"

    -------

    """
    socket_type = forge_socket.split("://")[0]
    parsed_socket = forge_socket.split("://")[1]
    if socket_type == 'unix':
        socket_target = '/'.join([
            'unix:/',
            forge_path, parsed_socket,
        ])
    else:
        socket_target = forge_socket
    return socket_target


def __parse_socket_grpc(forge_path, forge_socket):
    socket_addr = __parse_socket(forge_path, forge_socket)
    socket_type = socket_addr.split('://')[0]
    if socket_type == 'unix':
        return socket_addr
    elif socket_type == 'tcp':
        return socket_addr.split('://')[1]


def get_app_path():
    return expanduser(config['app']['path'])


def get_app_host():
    return config['app']['host']


def get_forge_path():
    return expanduser(config['forge']['path'])


def get_grpc_socket():
    return __parse_socket_grpc(
            get_forge_path(),
            config['forge']['sock_grpc'],
    )


def get_tcp_socket():
    return __parse_socket(
            get_app_path(),
            config['app']['sock_tcp'],
    )


def get_grpc_channel():
    return grpc.insecure_channel(get_grpc_socket())



import logging
import os.path as path
from time import sleep

from event_chain.application import app

from forge import ForgeConfig
from forge import ForgeSdk

logging.basicConfig(level=logging.DEBUG)
forgeSdk = ForgeSdk()
rpc = forgeSdk.rpc
logger = logging.getLogger("Test")


def wait():
    sleep(5)


def test():
    # bobb creates an event, alice buys a ticket, frank buys a ticket,
    # alice uses her ticket, checked by frank
    bobb = app.register_user('bobb', 'abcde1234')
    alice = app.register_user('alice', 'abcde1234')
    frank = app.register_user('frank', 'abcde1234')
    logger.info("User registration Done.")
    wait()

    assert (len(bobb.get_state().hosted) == 0)
    assert (len(alice.get_state().unused) == 0)
    assert (len(alice.get_state().used) == 0)
    assert (len(alice.get_state().participated) == 0)

    event_address = app.create_sample_event(bobb, 'Jay Chou Concert')
    logger.info("Event address is {}".format(event_address))
    app.refresh()

    logger.info("Buying first ticket...")
    ticket1_address = app.buy_ticket(event_address, alice)
    logger.info("First ticket is bought successfully.")
    app.refresh()

    assert (alice.get_state().unused[0] == ticket1_address)
    assert (len(alice.get_state().used) == 0)
    assert (len(alice.get_state().participated) == 0)

    logger.info("Buying second ticket...")
    ticket2_address = app.buy_ticket(event_address, frank)
    logger.info("Second ticket is bought successfully.")
    app.refresh()

    assert (frank.get_state().unused[0] == ticket2_address)
    assert (len(frank.get_state().used) == 0)
    assert (len(frank.get_state().participated) == 0)

    ticket1 = app.get_ticket_state(ticket1_address)
    ticket2 = app.get_ticket_state(ticket2_address)

    assert (ticket1.owner == alice.address)
    assert (ticket2.owner == frank.address)

    assert (not ticket1.is_used)
    assert (not ticket1.activated)

    # alice wants to use her ticket for event
    alice_tx = ticket1.gen_activate_asset_tx(alice)
    app.activate(alice_tx, frank)
    logger.info("Ticket has been activated!")
    app.refresh()

    ticket1 = app.get_ticket_state(ticket1_address)
    assert ticket1.is_used
    assert ticket1.activated
    assert (alice.get_state().used[0] == ticket1.address)
    assert (alice.get_state().participated[0] == event_address)
    assert (len(alice.get_state().unused) == 0)

    logger.info("Ticket is used successfully.")


def test_config():
    app_config = path.join(path.dirname(__file__), "config", "forge.toml")
    print(app_config)
    config = ForgeConfig(file_path=app_config)
    print(config.get_app_path())


if __name__ == "__main__":
    test_config()



import json
import logging
import os
import time

import event_chain.application.app as app
from event_chain import db

logger = logging.getLogger('ec-simulator')

DATA_PATH = os.path.join(os.path.dirname(__file__), "sample_data.json")

with open(DATA_PATH, "r") as read_file:
    data = json.load(read_file)


def set_up_db():
    db.init.initialized_app_folder()
    conn = db.init.reset_db()
    return conn


def simulate(conn):
    users = data.get('users')
    events = data.get('events')

    test_user = None

    logger.info("Creating Users...")
    for user in users:
        test_user = app.register_user(
            user.get('moniker'),
            user.get('passphrase'), conn,
        )

    time.sleep(5)
    logger.info("Users are created.")

    logger.info("Creating Events...")
    for event in events:
        app.create_event(test_user, conn, **event)
    logger.info("Events are created.")

    time.sleep(5)


if __name__ == '__main__':
    conn = set_up_db()
    logger.info("DB has been initialized.")

    simulate(conn)
    logger.info("Data has been simulated.")



from .abi_pb2 import *
from .enum_pb2 import *
from .rpc_pb2 import *
from .rpc_pb2_grpc import *
from .sdk_test_pb2 import *
from .service_pb2 import *
from .service_pb2_grpc import *
from .state_pb2 import *
from .tx_pb2 import *
from .type_pb2 import *
from . import vendor_pb2 as vendor



import ast
import json
import logging
import sqlite3
import sys
from time import sleep

import base58
import event_chain.application.app as app
import event_chain.config.config as config
import event_chain.db.utils as db
import requests
from event_chain.utils import helpers
from flask import flash
from flask import Flask
from flask import g
from flask import redirect
from flask import render_template
from flask import request
from flask import Response
from flask import session
from flask_googlemaps import GoogleMaps
from flask_qrcode import QRcode
from flask_wtf import FlaskForm
from wtforms import IntegerField
from wtforms import StringField
from wtforms import SubmitField
from wtforms import ValidationError
from wtforms import validators
from wtforms.validators import DataRequired

from flask_session import Session
from forge.utils import utils

application = Flask(__name__)
application.config['SECRET_KEY'] = 'hihihihihi'
application.config['WTF_CSRF_ENABLED'] = False
SESSION_TYPE = 'filesystem'
application.config[
    'GOOGLEMAPS_KEY'] = config.googlemaps_key
application.config.from_object(__name__)
Session(application)
QRcode(application)
GoogleMaps(application)

logging.basicConfig(level=logging.DEBUG)
DB_PATH = config.db_path
SERVER_ADDRESS = "http://" + config.app_host + ":5000/"

logging.info('DB: {}'.format(DB_PATH))
logging.info('forge port {}'.format(config.forge_config.get_grpc_socket()))
logging.info('app server address: {}'.format(SERVER_ADDRESS))

APP_SK = 'zyQBXZ7NijYQQRzLUryjkd1Mj4qSpofcnsgEh8a8ZrtbgM1jSKHrC85V' \
         'edZsQ5N3x5M298zpridcq2bKBZtmqroT'
APP_PK = "zExrfT2pXtVqdAqgZwjvdMBo5RpqSqn1fa43Wp93peuSR"
APP_ADDR = "z1UT9an1Z4W1gnmzASneER2J5eqtx5jfwgx"

ARC = 'https://arcwallet.io/i/'


def is_loggedin():
    if session.get('user'):
        return True
    else:
        return redirect('/login')


class EventForm(FlaskForm):
    title = StringField('Title', validators=[DataRequired()])
    confirm = SubmitField('Confirm')
    total = IntegerField('Total', validators=[DataRequired(),
                                              validators.number_range(min=1,
                                                                      max=30)
                                              ])
    start_time = StringField("StartTime", validators=[DataRequired()])
    description = StringField("Description")
    end_time = StringField("EndTime", validators=[DataRequired()])
    ticket_price = IntegerField("TicketPrice")
    address = StringField('Address')
    location = StringField('Location', validators=[DataRequired()])


def validate_name(form, field):
    for i in field.data:
        if i == ' ':
            raise ValidationError('Name should not contain white space.')
        if not i.isalnum():
            raise ValidationError(
                'Name should not contain special character',
            )


def validate_passphrase(form, field):
    has_alpha = False
    has_num = False
    for i in field.data:
        if i.isnumeric():
            has_num = True
        if i.isalpha():
            has_alpha = True
    if not has_alpha or not has_num:
        raise ValidationError(
            "Password must have both letters and numbers!",
        )


def verify_ticket(address):
    if not address:
        return response_error("Please provide a valid ticket address.")
    try:
        app.verify_ticket_address(address)
    except Exception as e:
        return response_error(e.args[0])


def verify_event(address):
    if not address:
        return response_error("Please provide a valid event address.")
    try:
        app.verify_event_address(address)
    except Exception as e:
        return response_error(e.args[0])


class RegisterForm(FlaskForm):
    name = StringField(
        'Name', validators=[
            DataRequired(),
            validators.length(min=4, max=20),
            validate_name,
        ],
    )
    passphrase = StringField(
        'Passphrase', validators=[
            DataRequired(),
            validate_passphrase,
        ],
    )
    confirm = SubmitField('Confirm')
    address = StringField('Address')


def wait():
    sleep(5)


def connect_db():
    return sqlite3.connect(DB_PATH)


@application.before_request
def before_request():
    g.db = connect_db()
    g.logger = logging.getLogger('app')
    g.logger.setLevel(level=logging.DEBUG)


@application.teardown_request
def teardown_request(exception):
    if hasattr(g, 'db'):
        g.db.close()


def gen_mobile_url(event_address):
    params = {
        'appPk': APP_PK,
        'appDid': 'did:abt:' + APP_ADDR,
        'action': 'requestAuth',
        'url': SERVER_ADDRESS + 'api/mobile-buy-ticket/{}'.format(
            event_address,
        ),
    }
    r = requests.Request('GET', ARC, params=params).prepare()
    g.logger.info(u'Url generated {}'.format(r.url))
    return r.url


def gen_consume_url(event_address):
    url = SERVER_ADDRESS + 'api/mobile-require-asset/{}'.format(
        event_address,
    )
    return gen_did_url(url, 'RequestAuth')


def gen_did_url(url, action):
    g.logger.debug(
        "Generating url for DID call. Call back url provided is {}".format(
            url,
        ),
    )
    params = {
        'appPk': APP_PK,
        'appDid': 'did:abt:' + APP_ADDR,
        'action': action,
        'url': url,
    }
    r = requests.Request('GET', ARC, params=params).prepare()
    return r.url


@application.route("/details/<address>", methods=['GET', 'POST'])
def event_detail(address):
    error = verify_event(address)
    if error:
        return error
    forge_web = 'http://' + config.app_host + ':8211/node/explorer/txs/'

    event = app.get_event_state(address)
    host = app.get_participant_state(event.owner)
    form = EventForm()
    if is_loggedin():
        url = gen_mobile_url(address)
        g.logger.info("Url for mobile buy ticket: {}".format(url))

        consume_url = gen_consume_url(address)
        g.logger.info("Url for mobile consume ticket: {}".format(consume_url))

        txs = app.list_event_exchange_txs(g.db, address)
        num_txs = len(txs)
        tx_lists = chunks(txs, 3)
        g.logger.debug('forgeweb:{}'.format(forge_web))
        return render_template(
            'event_details.html', **locals(),
            to_display_time=helpers.to_display_time
        )
    return redirect('/login')


@application.route("/ticket-detail", methods=['GET', 'POST'])
def ticket_detail():
    form = EventForm()
    address = form.address.data if form.address.data else request.args.get(
        'address',
    )
    view_only = request.args.get('viewonly', False)
    error = verify_ticket(address)
    if error:
        return error
    ticket = app.get_ticket_state(address)
    event = app.get_event_state(ticket.event_address)
    host = app.get_participant_state(event.owner)

    return render_template(
        "ticket_details.html", ticket=ticket, event=event, form=form,
        host=host, view_only=view_only,
    )


@application.route("/buy", methods=['POST'])
def buy():
    if not session.get('user'):
        return redirect('/login')
    refresh_token()
    form = EventForm()
    address = form.address.data
    event = app.get_event_state(address)

    error = verify_event(address)
    if error:
        return error

    hash = app.buy_ticket(address, session.get('user'), g.db)
    if not hash:
        g.logger.error("Fail to buy ticket from web.")
        flash('Oops! Someone is faster than you. Get another ticket!')
        return redirect('/')
    else:
        db.insert_exchange_tx(g.db, event.address, hash)
        g.logger.info("ticket is bought successfully from web.")
        flash(
            'Congratulations! Ticket for Event "{}" is bought '
            'successfully!'.format(
                event.event_info.title), category='info')
    return redirect('/tickets')


@application.route("/activate/<address>", methods=['GET', 'POST'])
def use(address):
    refresh_token()

    error = verify_ticket(address)
    if error:
        return error
    app.consume(address, session['user'])
    sleep(2)
    flash('Your ticket has been checked successfully! Enjoy your event!',
          category='info')
    return redirect('/tickets')


@application.route("/", methods=['GET', 'POST'])
def event_list():
    events = app.list_events(g.db)
    event_lists = chunks(events, 3)
    return render_template(
        'event_list.html', event_lists=event_lists,
        session=session, number=len(events),
    )


def get_event_for_ticket(tickets):
    res = {}
    for ticket in tickets:
        event_address = ticket.ticket_info.event_address
        event = app.get_event_state(event_address)
        res[ticket.address] = event
    return res


@application.route("/logout", methods=['GET', 'POST'])
def logout():
    session['user'] = None
    db.delete_mobile_address(g.db)
    return redirect('/')


@application.route("/tickets")
def ticket_list():
    if not session.get('user'):
        return redirect('login')
    tickets = app.list_unused_tickets(session.get('user').address)
    num_tickets = len(tickets)
    user = app.get_participant_state(session.get('user').address)
    events = get_event_for_ticket(tickets)
    ticket_lists = chunks(tickets, 3)
    return render_template(
        'tickets.html', ticket_lists=ticket_lists, events=events,
        user=user, view_only=False, num_tickets=num_tickets
    )


@application.context_processor
def inject_mobile_address():
    return dict(mobile_address=db.get_last_mobile_address(g.db))


@application.route("/mobile-account", methods=['GET'])
def mobile_account():
    address = db.get_last_mobile_address(g.db)
    if not address:
        flash("Please use your mobile wallet to buy a ticket first!",
              category='info')
        return redirect('/')
    tickets = app.list_unused_tickets(address)
    user = app.get_participant_state(address)
    events = get_event_for_ticket(tickets)
    ticket_lists = chunks(tickets, 3)
    return render_template(
        'tickets.html', ticket_lists=ticket_lists, events=events,
        user=user, view_only=True
    )


@application.route("/create", methods=['GET', 'POST'])
def create_event():
    if not session.get('user'):
        return redirect('/login')
    refresh_token()
    form = EventForm()
    if form.validate_on_submit():
        if request.method == "POST":
            app.create_event(
                user=session.get('user'),
                title=form.title.data,
                total=form.total.data,
                description=form.description.data,
                start_time=form.start_time.data,
                end_time=form.end_time.data,
                ticket_price=form.ticket_price.data * 1e+16,
                location=form.location.data,
                conn=g.db,
            )
            wait()
            return redirect('/')
    else:
        g.logger.error(form.errors)
        flash_errors(form)
    return render_template('event_create.html', form=form)


def refresh_token():
    user = session.get('user')
    user = app.load_user(
        moniker=user.moniker,
        passphrase=user.passphrase,
        conn=g.db,
        address=user.address,
    )
    session['user'] = user
    g.logger.info("Token refreshed!")


def flash_errors(form):
    for field, errors in form.errors.items():
        for error in errors:
            flash(
                "Error in the {} field - {}".format(
                    getattr(form, field).label.text,
                    error), 'error',
            )


@application.route("/login", methods=['GET', 'POST'])
def login():
    form = RegisterForm()
    if form.validate_on_submit():
        user = app.load_user(
            moniker=form.name.data,
            passphrase=form.passphrase.data,
            address=form.address.data,
        )
        session['user'] = user
        return redirect('/')
    else:
        flash_errors(form)
    return render_template('login.html', form=form)


@application.route("/register", methods=['GET', 'POST'])
def register():
    form = RegisterForm()
    if form.validate_on_submit():
        user = app.register_user(
            form.name.data,
            form.passphrase.data,
            g.db,
        )
        session['user'] = user
        g.logger.debug(
            'New User registered! wallet: {}, token: {}'.format(
                user.wallet,
                user.token,
            ),
        )
        g.logger.debug("form is validated!!")
        wait()
    else:
        flash_errors(form)
        return render_template('login.html', form=form)
    return redirect('/')


def send_did_request(
        url, description, endpoint, workflow, tx=None,
        target=None,
):
    if tx:
        base58_encoded = (b'z' + base58.b58encode(
            tx.SerializeToString(),
        )).decode()
        g.logger.debug(
            u"Sending request to DID with base58 encoded tx: {} and"
            u" url {}".format(base58_encoded, url),
        )
    else:
        base58_encoded = None

    params = {
        'sk': APP_SK,
        'pk': APP_PK,
        'address': APP_ADDR,
        'tx': base58_encoded,
        'description': description,
        'target': target,
        'url': url,
        'workflow': workflow,
    }
    headers = {'content-type': 'application/json'}
    call_url = 'http://localhost:4000/api/' + endpoint
    g.logger.debug('call url : {}'.format(call_url))
    response = requests.post(
        call_url,
        json=params,
        headers=headers,
    )
    g.logger.info("Response from did: {}".format(response.content.decode()))
    return Response(
        response.content.decode(), status=200,
        mimetype='application/json',
    )


@application.route(
    "/api/mobile-buy-ticket/<event_address>",
    methods=['GET', 'POST'],
)
def mobile_buy_ticket(event_address):
    try:
        error = verify_event(event_address)
        if error:
            return error
        if request.method == 'GET':
            g.logger.debug("Receives get request for mobile-buy-ticket.")
            user_did = request.args.get('userDid')
            user_pk = utils.multibase_b58decode(request.args.get('userPk'))
            if not user_did:
                return response_error("Please provide a valid user did.")
            user_address = user_did.split(":")[2]
            g.logger.debug(
                "user address parsed from request {}".format(user_address),
            )

            event = app.get_event_state(event_address)
            updated_exchange_tx = helpers.update_tx_multisig(
                event.get_exchange_tx(),
                user_address,
                user_pk
            )
            g.logger.debug('new tx {}:'.format(updated_exchange_tx))
            call_back_url = SERVER_ADDRESS + "api/mobile-buy-ticket/"
            des = 'Confirm the purchase below.'

            did_request_params = {
                'user_did': user_did,
                'tx': updated_exchange_tx,
                'url': call_back_url,
                'description': des,
                'action': 'responseAuth',
                'workflow': 'buy-ticket',
            }
            response = app.did_auth_require_multisig(**did_request_params)
            g.logger.debug('did auth response: {}'.format(response))
            return response

        elif request.method == 'POST':
            g.logger.debug("Receives post request for mobile-buy-ticket.")
            try:
                req_data = request.get_data(as_text=True)
                g.logger.debug("Receives data from wallet {}".format(req_data))
                req = ast.literal_eval(req_data)
                wallet_response = helpers.WalletResponse(req)
            except Exception as e:
                g.logger.error(e, exc_info=True)
                return response_error("Error in parsing wallet data.")

            user_address = wallet_response.get_address()
            participant_state = app.get_participant_state(user_address)
            if not participant_state:
                return response_error(
                    "user {} doesn't exist.".format(user_address))

            ticket_address, hash = app.buy_ticket_mobile(
                event_address,
                user_address,
                wallet_response.get_signature(),
                wallet_response.get_user_pk(),
            )

            if ticket_address and hash:
                g.logger.info("Ticket {} is bought successfully "
                              "by mobile.".format(ticket_address))

                db.insert_mobile_address(g.db, user_address)
                db.insert_exchange_tx(g.db, event_address, hash)
                base58_encoded = helpers.base58_encode_tx(
                    helpers.update_tx_multisig(
                        app.get_event_state(
                            event_address).get_exchange_tx(),
                        user_address,
                        wallet_response.get_user_pk(),
                    ))
                js = json.dumps({'ticket': ticket_address,
                                 'hash': hash,
                                 'tx': base58_encoded})
                g.logger.debug('success response: {}'.format(str(js)))
                resp = Response(js, status=200, mimetype='application/json')
                return resp
            else:
                g.logger.error(
                    'Fail to buy ticket. ticket '
                    'address: {0}, hash : {1}.'.format(ticket_address,
                                                       hash))
                return response_error('Please try buying the ticket again.')
    except Exception as e:
        g.logger.error(e, exc_info=True)
        return response_error('Exception in buying ticket.')


def response_error(msg):
    error = json.dumps({'error': msg})
    return Response(error, status=400, mimetype='application/json')


@application.route("/error", methods=['GET', 'POST'])
def error():
    return "Sorry there's something wrong in purchasing your ticket."


@application.route(
    "/api/mobile-require-asset/<event_address>",
    methods=['GET', 'POST'],
)
def mobile_require_asset(event_address):
    try:
        error = verify_event(event_address)
        if error:
            return error
        event = app.get_event_state(event_address)

        if request.method == 'GET':
            call_back_url = SERVER_ADDRESS + "api/mobile-require-asset/"
            des = 'Select a ticket for event.'
            target = event.event_info.title
            did_request_params = {
                'url': call_back_url,
                'description': des,
                'action': 'responseAuth',
                'workflow': 'use-ticket',
                'target': target,
            }
            response = app.did_auth_require_asset(**did_request_params)
            return response

        if request.method == 'POST':
            try:
                req_data = request.get_data(as_text=True)
                g.logger.debug("Receives data from wallet {}".format(req_data))
                req = ast.literal_eval(req_data)
                wallet_response = helpers.WalletResponse(req)
            except Exception as e:
                g.logger.error(
                    "error in parsing wallet data, error:{}".format(e),
                )
                g.logger.error(e, exc_info=True)
                return response_error("Error in parsing wallet data. Original "
                                      "data received is {}".format(req))

            asset_address = wallet_response.get_asset_address()
            if not asset_address:
                g.logger.error(
                    "No available asset address in wallet response.",
                )
                return response_error("Please provide an asset address.")

            error = verify_ticket(asset_address)
            if error:
                g.logger.error(
                    "ticket address in wallet response is not valid.",
                )
                return response_error("Please provide a valid ticket address.")

            user_address = wallet_response.get_address()
            call_back_url = SERVER_ADDRESS + "api/mobile-consume/{}".format(
                asset_address,
            )
            des = 'Confirm to use the ticket.'
            consume_tx = event.event_info.consume_tx
            multisig_data = helpers.encode_string_to_any(
                'fg:x:address',
                asset_address,
            )

            new_tx = helpers.update_tx_multisig(
                tx=consume_tx, signer=user_address,
                pk=wallet_response.get_user_pk(),
                data=multisig_data,
            )

            did_request_params = {
                'user_did': user_address,
                'tx': new_tx,
                'url': call_back_url,
                'description': des,
                'action': 'responseAuth',
                'workflow': 'use-ticket',
            }
            response = app.did_auth_require_multisig(**did_request_params)
            return response

    except Exception as e:
        g.logger.error(e, exc_info=True)
        return response_error("Exception in requesting asset.")


@application.route(
    "/api/mobile-consume/<ticket_address>", methods=['POST'],
)
def mobile_consume(ticket_address):
    try:
        error = verify_ticket(ticket_address)
        if error:
            return error

        ticket = app.get_ticket_state(ticket_address)
        event = app.get_event_state(ticket.event_address)

        if request.method == 'POST':
            try:
                req_data = request.get_data(as_text=True)
                g.logger.debug("Receives data from wallet {}".format(req_data))
                req = ast.literal_eval(req_data)
                wallet_response = helpers.WalletResponse(req)
            except Exception as e:
                g.logger.error(e, exc_info=True)
                return response_error("Error in parsing wallet data. Original "
                                      "data received is {}".format(req))

            hash = app.consume_ticket_mobile(
                ticket,
                event.event_info.consume_tx,
                wallet_response.get_address(),
                wallet_response.get_signature(),
                wallet_response.get_user_pk(),
            )
            multisig_data = helpers.encode_string_to_any(
                'fg:x:address',
                ticket_address,
            )
            base58_tx = helpers.base58_encode_tx(helpers.update_tx_multisig(
                tx=event.event_info.consume_tx,
                signer=wallet_response.get_address(),
                pk=wallet_response.get_user_pk(),
                data=multisig_data
            ))
            if hash:
                g.logger.info("ConsumeTx has been sent.")
                js = json.dumps({'hash': hash,
                                 'tx': base58_tx})
                g.logger.debug('success response: {}'.format(js))
                resp = Response(js, status=200, mimetype='application/json')
                return resp
            else:
                g.logger.error('Fail to consume ticket.')
                return response_error('Your ticket might have been '
                                      'checked out before. '
                                      'Please wait and try again.')
    except Exception as e:
        g.logger.error(e, exc_info=True)
        return response_error("Exception in consuming ticket.")


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i:i + n]


if __name__ == '__main__':
    run_type = sys.argv[1]
    application.jinja_env.auto_reload = True
    application.config['TEMPLATES_AUTO_RELOAD'] = True
    if run_type == 'debug':
        application.run(debug=True, host='0.0.0.0')
    else:
        application.run(
            debug=False, host='0.0.0.0'
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sdk_test.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='sdk_test.proto',
  package='forge_abi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0esdk_test.proto\x12\tforge_abi\x1a\x19google/protobuf/any.proto\"(\n\x0bpythonSDKTx\x12\n\n\x02to\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\rb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_PYTHONSDKTX = _descriptor.Descriptor(
  name='pythonSDKTx',
  full_name='forge_abi.pythonSDKTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='to', full_name='forge_abi.pythonSDKTx.to', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='forge_abi.pythonSDKTx.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=96,
)

DESCRIPTOR.message_types_by_name['pythonSDKTx'] = _PYTHONSDKTX
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

pythonSDKTx = _reflection.GeneratedProtocolMessageType('pythonSDKTx', (_message.Message,), dict(
  DESCRIPTOR = _PYTHONSDKTX,
  __module__ = 'sdk_test_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.pythonSDKTx)
  ))
_sym_db.RegisterMessage(pythonSDKTx)


# @@protoc_insertion_point(module_scope)



from enum import Enum


class SupportedActions(Enum):
    VERIFY_TX = 'verify_tx'
    UPDATE_STATE = 'update_state'

    @classmethod
    def has_action(cls, value):
        return any(value == item.value for item in cls)


class TxHandler:
    def __init__(self, tx_type, verify_tx_func, update_state_func):
        self.tx_type = tx_type
        self.verify_tx = verify_tx_func
        self.update_state = update_state_func



import logging
from inspect import getmembers
from inspect import isfunction

from forge import protos
from forge import utils
from forge.rpc import chain
from forge.rpc import event
from forge.rpc import file
from forge.rpc import state
from forge.rpc import statistic
from forge.rpc import wallet

logger = logging.getLogger('forge-rpc')


def __get_module_functions(module):
    return {item[0]: getattr(module, item[0]) for item in
            getmembers(module, isfunction)}


__all_services = {**__get_module_functions(chain),
                  **__get_module_functions(state),
                  **__get_module_functions(event),
                  **__get_module_functions(file),
                  **__get_module_functions(wallet),
                  **__get_module_functions(statistic)}

for name, func in __all_services.items():
    vars()[name] = func


__wallet_type = protos.WalletType(pk=0, hash=1, address=1)


def send_itx(type_url, itx, wallet, token, nonce=1):
    encoded_itx = utils.encode_to_any(type_url, itx)
    tx = create_tx(
        itx=encoded_itx, from_address=wallet.address,
        wallet=wallet, token=token, nonce=nonce
    )
    return send_tx(tx.tx)


def create_asset(type_url, asset, wallet, token):
    encoded_asset = utils.encode_to_any(type_url, asset)
    create_asset_itx = utils.encode_to_any(
        type_url='fg:t:create_asset',
        data=protos.CreateAssetTx(data=encoded_asset),
    )
    tx = create_tx(
        create_asset_itx, wallet.address, wallet, token,
    )
    return send_tx(tx.tx)


def update_asset(type_url, address, asset, wallet, token):
    encoded_asset = utils.encode_to_any(type_url, asset)
    update_asset_itx = utils.encode_to_any(
        type_url='fg:t:update_asset',
        data=protos.UpdateAssetTx(
            address=address,
            data=encoded_asset,
        ),
    )
    tx = create_tx(
        update_asset_itx, wallet.address, wallet, token,
    )
    return send_tx(tx.tx)


def get_single_account_state(address):
    if address:
        accounts = get_account_state({'address': address})
        account = next(accounts)
        if utils.is_proto_empty(account):
            return None
        else:
            return account.state


def get_single_tx_info(hash):
    if hash:
        infos = get_tx(hash)
        info = next(infos)
        if utils.is_proto_empty(info):
            return None
        else:
            return info.info


def get_single_asset_state(address):
    if address:
        assets = get_asset_state({'address': address})
        asset = next(assets)
        if not utils.is_proto_empty(asset):
            return asset.state


def get_nonce(address):
    account = get_single_account_state(address)
    return account.nonce


def multisig_consume_asset_tx(tx, wallet, token, data):
    return multisig(tx, wallet, token, data=data)



import logging
import os
from socket import AF_INET
from socket import AF_UNIX
from socket import SOCK_STREAM
from socket import socket

from forge import protos
from forge.utils import utils
from forge.utils.helper import SupportedActions


class ForgeServer:
    def __init__(
            self, handlers, address,
            buf_size=1500,
    ):
        self.server, self.address = ForgeServer.__parse_socket_address(address)
        self.buf_size = buf_size
        self.handlers = {handler.tx_type: handler for handler in handlers}
        self.buffer = b''
        self.active = True
        self.conn = None
        self.response_type = {
            SupportedActions.VERIFY_TX: protos.ResponseVerifyTx,
            SupportedActions.UPDATE_STATE: protos.ResponseUpdateState,
        }

        self.logger = logging.getLogger(__name__)

    def register_handler(self, handler):
        self.handlers[handler.tx_type] = handler

    def start(self):
        self.server.bind(self.address)
        self.server.listen(1)
        self.logger.debug("socket type is : {}".format(self.server))
        self.logger.info(
            "server has binded to address {}".format(self.address),
        )
        while self.active:
            self.logger.info("Waiting for connection...")
            self.conn, addr = self.server.accept()
            self.logger.info("Connected!")

            while True:
                data = self.conn.recv(1024)
                self.logger.debug("bytes received: {}".format(data))
                if data:
                    self.buffer = self.buffer + data
                    self.__process_buffer()
                else:
                    break

    def __process_buffer(self):
        while self.__buffer_contains_full_request():
            request_bytes, len, pos = utils.decode(
                self.buffer,
            )
            request = utils.parse_to_proto(request_bytes, protos.Request)
            action = request.WhichOneof('value')
            self.__handle_request(request, action)
            self.__update_buffer(pos, len)

    def __update_buffer(self, start, length):
        self.buffer = self.buffer[start + length:]

    def __handle_request(self, request, action):
        self.logger.debug("EC Received a {} request!".format(action))

        action_request = getattr(request, action)
        if action == 'info':
            self.__handle_info_request()
        else:
            itx_type = action_request.tx.itx.type_url
            if self.__is_type_supported(itx_type):
                func = self.__get_itx_action(itx_type, action)
                res = self.__reply(
                    response=func(
                        action_request,
                    ), action=action,
                )
                self.conn.send(res)
            else:
                self.conn.send(self.__reply(action=action, unsupported=True))

        self.logger.debug("type {} has been processed!".format(action))

    def __handle_info_request(self):
        url_list = [key for key in self.handlers.keys()]
        response = utils.encode(
            protos.Response(info=protos.ResponseInfo(type_urls=url_list)),
        )
        self.logger.debug('type_urls: {}'.format(url_list))
        self.conn.send(response)

    def __is_type_supported(self, itx_type):
        return itx_type in self.handlers.keys()

    def __get_itx_action(self, itx_type, action):
        return getattr(self.handlers.get(itx_type), action)

    def stop(self):
        self.active = False

    @staticmethod
    def __parse_socket_address(address):
        socket_type = address.split('://')[0]
        socket_addr = address.split('://')[1]

        if socket_type == 'tcp':
            parsed_addr = socket_addr.split(":")
            return socket(AF_INET, SOCK_STREAM), (
                parsed_addr[0], int(parsed_addr[1]),
            )
        elif socket_type == 'unix':
            if os.path.exists(socket_addr):
                os.unlink(socket_addr)
            return socket(AF_UNIX, SOCK_STREAM), socket_addr
        else:
            raise AttributeError(
                "The socket in Forge.toml should start with either tcp "
                "or unix",
            )

    def __buffer_contains_full_request(self):
        if len(self.buffer) > 0:
            req, req_len, start_pos = utils.decode(self.buffer)
            if req_len + start_pos <= len(self.buffer):
                self.logger.debug(
                    "Buffer contains a full request. Processing the "
                    "request..",
                )
                return True
        return False

    def __reply(self, action, unsupported=False, response=None):
        if unsupported:
            result = protos.Response(
                **{action: self.response_type[action](
                    code=protos.StatusCode.UNSUPPORTED_TX,
                )}
            )
            self.logger.debug("Receives unsupported action".format(action))
        else:
            result = protos.Response(**{action: response})
        return utils.encode(result)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import rpc_pb2 as rpc__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='service.proto',
  package='forge_abi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\rservice.proto\x12\tforge_abi\x1a\trpc.proto2\x87\t\n\x08\x43hainRpc\x12\x44\n\tcreate_tx\x12\x1a.forge_abi.RequestCreateTx\x1a\x1b.forge_abi.ResponseCreateTx\x12\x43\n\x08multisig\x12\x1a.forge_abi.RequestMultisig\x1a\x1b.forge_abi.ResponseMultisig\x12>\n\x07send_tx\x12\x18.forge_abi.RequestSendTx\x1a\x19.forge_abi.ResponseSendTx\x12?\n\x06get_tx\x12\x17.forge_abi.RequestGetTx\x1a\x18.forge_abi.ResponseGetTx(\x01\x30\x01\x12H\n\tget_block\x12\x1a.forge_abi.RequestGetBlock\x1a\x1b.forge_abi.ResponseGetBlock(\x01\x30\x01\x12G\n\nget_blocks\x12\x1b.forge_abi.RequestGetBlocks\x1a\x1c.forge_abi.ResponseGetBlocks\x12`\n\x13get_unconfirmed_txs\x12#.forge_abi.RequestGetUnconfirmedTxs\x1a$.forge_abi.ResponseGetUnconfirmedTxs\x12Q\n\x0eget_chain_info\x12\x1e.forge_abi.RequestGetChainInfo\x1a\x1f.forge_abi.ResponseGetChainInfo\x12N\n\rget_node_info\x12\x1d.forge_abi.RequestGetNodeInfo\x1a\x1e.forge_abi.ResponseGetNodeInfo\x12=\n\x06search\x12\x18.forge_abi.RequestSearch\x1a\x19.forge_abi.ResponseSearch\x12K\n\x0cget_net_info\x12\x1c.forge_abi.RequestGetNetInfo\x1a\x1d.forge_abi.ResponseGetNetInfo\x12`\n\x13get_validators_info\x12#.forge_abi.RequestGetValidatorsInfo\x1a$.forge_abi.ResponseGetValidatorsInfo\x12G\n\nget_config\x12\x1b.forge_abi.RequestGetConfig\x1a\x1c.forge_abi.ResponseGetConfig\x12Z\n\x11get_asset_address\x12!.forge_abi.RequestGetAssetAddress\x1a\".forge_abi.ResponseGetAssetAddress\x12\x44\n\tsign_data\x12\x1a.forge_abi.RequestSignData\x1a\x1b.forge_abi.ResponseSignData2\xa2\x01\n\x08\x45ventRpc\x12H\n\tsubscribe\x12\x1b.forge_abi.RequestSubscribe\x1a\x1c.forge_abi.ResponseSubscribe0\x01\x12L\n\x0bunsubscribe\x12\x1d.forge_abi.RequestUnsubscribe\x1a\x1e.forge_abi.ResponseUnsubscribe2\xdf\x01\n\x07\x46ileRpc\x12I\n\nstore_file\x12\x1b.forge_abi.RequestStoreFile\x1a\x1c.forge_abi.ResponseStoreFile(\x01\x12\x46\n\tload_file\x12\x1a.forge_abi.RequestLoadFile\x1a\x1b.forge_abi.ResponseLoadFile0\x01\x12\x41\n\x08pin_file\x12\x19.forge_abi.RequestPinFile\x1a\x1a.forge_abi.ResponsePinFile2\xf4\x02\n\x08StateRpc\x12^\n\x11get_account_state\x12!.forge_abi.RequestGetAccountState\x1a\".forge_abi.ResponseGetAccountState(\x01\x30\x01\x12X\n\x0fget_asset_state\x12\x1f.forge_abi.RequestGetAssetState\x1a .forge_abi.ResponseGetAssetState(\x01\x30\x01\x12X\n\x0fget_stake_state\x12\x1f.forge_abi.RequestGetStakeState\x1a .forge_abi.ResponseGetStakeState(\x01\x30\x01\x12T\n\x0fget_forge_state\x12\x1f.forge_abi.RequestGetForgeState\x1a .forge_abi.ResponseGetForgeState2\xed\x03\n\tWalletRpc\x12P\n\rcreate_wallet\x12\x1e.forge_abi.RequestCreateWallet\x1a\x1f.forge_abi.ResponseCreateWallet\x12J\n\x0bload_wallet\x12\x1c.forge_abi.RequestLoadWallet\x1a\x1d.forge_abi.ResponseLoadWallet\x12S\n\x0erecover_wallet\x12\x1f.forge_abi.RequestRecoverWallet\x1a .forge_abi.ResponseRecoverWallet\x12L\n\x0blist_wallet\x12\x1c.forge_abi.RequestListWallet\x1a\x1d.forge_abi.ResponseListWallet0\x01\x12P\n\rremove_wallet\x12\x1e.forge_abi.RequestRemoveWallet\x1a\x1f.forge_abi.ResponseRemoveWallet\x12M\n\x0c\x64\x65\x63lare_node\x12\x1d.forge_abi.RequestDeclareNode\x1a\x1e.forge_abi.ResponseDeclareNode2\x9e\x06\n\x0cStatisticRpc\x12\x63\n\x14get_forge_statistics\x12$.forge_abi.RequestGetForgeStatistics\x1a%.forge_abi.ResponseGetForgeStatistics\x12\\\n\x11list_transactions\x12\".forge_abi.RequestListTransactions\x1a#.forge_abi.ResponseListTransactions\x12G\n\nget_assets\x12\x1b.forge_abi.RequestGetAssets\x1a\x1c.forge_abi.ResponseGetAssets\x12G\n\nget_stakes\x12\x1b.forge_abi.RequestGetStakes\x1a\x1c.forge_abi.ResponseGetStakes\x12W\n\x10get_top_accounts\x12 .forge_abi.RequestGetTopAccounts\x1a!.forge_abi.ResponseGetTopAccounts\x12l\n\x17list_asset_transactions\x12\'.forge_abi.RequestListAssetTransactions\x1a(.forge_abi.ResponseListAssetTransactions\x12J\n\x0blist_blocks\x12\x1c.forge_abi.RequestListBlocks\x1a\x1d.forge_abi.ResponseListBlocks\x12J\n\x0blist_assets\x12\x1c.forge_abi.RequestListAssets\x1a\x1d.forge_abi.ResponseListAssets\x12Z\n\x11get_health_status\x12!.forge_abi.RequestGetHealthStatus\x1a\".forge_abi.ResponseGetHealthStatusb\x06proto3')
  ,
  dependencies=[rpc__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_CHAINRPC = _descriptor.ServiceDescriptor(
  name='ChainRpc',
  full_name='forge_abi.ChainRpc',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=40,
  serialized_end=1199,
  methods=[
  _descriptor.MethodDescriptor(
    name='create_tx',
    full_name='forge_abi.ChainRpc.create_tx',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTCREATETX,
    output_type=rpc__pb2._RESPONSECREATETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='multisig',
    full_name='forge_abi.ChainRpc.multisig',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTMULTISIG,
    output_type=rpc__pb2._RESPONSEMULTISIG,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='send_tx',
    full_name='forge_abi.ChainRpc.send_tx',
    index=2,
    containing_service=None,
    input_type=rpc__pb2._REQUESTSENDTX,
    output_type=rpc__pb2._RESPONSESENDTX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_tx',
    full_name='forge_abi.ChainRpc.get_tx',
    index=3,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETTX,
    output_type=rpc__pb2._RESPONSEGETTX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_block',
    full_name='forge_abi.ChainRpc.get_block',
    index=4,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETBLOCK,
    output_type=rpc__pb2._RESPONSEGETBLOCK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_blocks',
    full_name='forge_abi.ChainRpc.get_blocks',
    index=5,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETBLOCKS,
    output_type=rpc__pb2._RESPONSEGETBLOCKS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_unconfirmed_txs',
    full_name='forge_abi.ChainRpc.get_unconfirmed_txs',
    index=6,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETUNCONFIRMEDTXS,
    output_type=rpc__pb2._RESPONSEGETUNCONFIRMEDTXS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_chain_info',
    full_name='forge_abi.ChainRpc.get_chain_info',
    index=7,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETCHAININFO,
    output_type=rpc__pb2._RESPONSEGETCHAININFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_node_info',
    full_name='forge_abi.ChainRpc.get_node_info',
    index=8,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETNODEINFO,
    output_type=rpc__pb2._RESPONSEGETNODEINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='search',
    full_name='forge_abi.ChainRpc.search',
    index=9,
    containing_service=None,
    input_type=rpc__pb2._REQUESTSEARCH,
    output_type=rpc__pb2._RESPONSESEARCH,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_net_info',
    full_name='forge_abi.ChainRpc.get_net_info',
    index=10,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETNETINFO,
    output_type=rpc__pb2._RESPONSEGETNETINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_validators_info',
    full_name='forge_abi.ChainRpc.get_validators_info',
    index=11,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETVALIDATORSINFO,
    output_type=rpc__pb2._RESPONSEGETVALIDATORSINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_config',
    full_name='forge_abi.ChainRpc.get_config',
    index=12,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETCONFIG,
    output_type=rpc__pb2._RESPONSEGETCONFIG,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_asset_address',
    full_name='forge_abi.ChainRpc.get_asset_address',
    index=13,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETASSETADDRESS,
    output_type=rpc__pb2._RESPONSEGETASSETADDRESS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='sign_data',
    full_name='forge_abi.ChainRpc.sign_data',
    index=14,
    containing_service=None,
    input_type=rpc__pb2._REQUESTSIGNDATA,
    output_type=rpc__pb2._RESPONSESIGNDATA,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_CHAINRPC)

DESCRIPTOR.services_by_name['ChainRpc'] = _CHAINRPC


_EVENTRPC = _descriptor.ServiceDescriptor(
  name='EventRpc',
  full_name='forge_abi.EventRpc',
  file=DESCRIPTOR,
  index=1,
  serialized_options=None,
  serialized_start=1202,
  serialized_end=1364,
  methods=[
  _descriptor.MethodDescriptor(
    name='subscribe',
    full_name='forge_abi.EventRpc.subscribe',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTSUBSCRIBE,
    output_type=rpc__pb2._RESPONSESUBSCRIBE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unsubscribe',
    full_name='forge_abi.EventRpc.unsubscribe',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTUNSUBSCRIBE,
    output_type=rpc__pb2._RESPONSEUNSUBSCRIBE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_EVENTRPC)

DESCRIPTOR.services_by_name['EventRpc'] = _EVENTRPC


_FILERPC = _descriptor.ServiceDescriptor(
  name='FileRpc',
  full_name='forge_abi.FileRpc',
  file=DESCRIPTOR,
  index=2,
  serialized_options=None,
  serialized_start=1367,
  serialized_end=1590,
  methods=[
  _descriptor.MethodDescriptor(
    name='store_file',
    full_name='forge_abi.FileRpc.store_file',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTSTOREFILE,
    output_type=rpc__pb2._RESPONSESTOREFILE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='load_file',
    full_name='forge_abi.FileRpc.load_file',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLOADFILE,
    output_type=rpc__pb2._RESPONSELOADFILE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='pin_file',
    full_name='forge_abi.FileRpc.pin_file',
    index=2,
    containing_service=None,
    input_type=rpc__pb2._REQUESTPINFILE,
    output_type=rpc__pb2._RESPONSEPINFILE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_FILERPC)

DESCRIPTOR.services_by_name['FileRpc'] = _FILERPC


_STATERPC = _descriptor.ServiceDescriptor(
  name='StateRpc',
  full_name='forge_abi.StateRpc',
  file=DESCRIPTOR,
  index=3,
  serialized_options=None,
  serialized_start=1593,
  serialized_end=1965,
  methods=[
  _descriptor.MethodDescriptor(
    name='get_account_state',
    full_name='forge_abi.StateRpc.get_account_state',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETACCOUNTSTATE,
    output_type=rpc__pb2._RESPONSEGETACCOUNTSTATE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_asset_state',
    full_name='forge_abi.StateRpc.get_asset_state',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETASSETSTATE,
    output_type=rpc__pb2._RESPONSEGETASSETSTATE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_stake_state',
    full_name='forge_abi.StateRpc.get_stake_state',
    index=2,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETSTAKESTATE,
    output_type=rpc__pb2._RESPONSEGETSTAKESTATE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_forge_state',
    full_name='forge_abi.StateRpc.get_forge_state',
    index=3,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETFORGESTATE,
    output_type=rpc__pb2._RESPONSEGETFORGESTATE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STATERPC)

DESCRIPTOR.services_by_name['StateRpc'] = _STATERPC


_WALLETRPC = _descriptor.ServiceDescriptor(
  name='WalletRpc',
  full_name='forge_abi.WalletRpc',
  file=DESCRIPTOR,
  index=4,
  serialized_options=None,
  serialized_start=1968,
  serialized_end=2461,
  methods=[
  _descriptor.MethodDescriptor(
    name='create_wallet',
    full_name='forge_abi.WalletRpc.create_wallet',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTCREATEWALLET,
    output_type=rpc__pb2._RESPONSECREATEWALLET,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='load_wallet',
    full_name='forge_abi.WalletRpc.load_wallet',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLOADWALLET,
    output_type=rpc__pb2._RESPONSELOADWALLET,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='recover_wallet',
    full_name='forge_abi.WalletRpc.recover_wallet',
    index=2,
    containing_service=None,
    input_type=rpc__pb2._REQUESTRECOVERWALLET,
    output_type=rpc__pb2._RESPONSERECOVERWALLET,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_wallet',
    full_name='forge_abi.WalletRpc.list_wallet',
    index=3,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLISTWALLET,
    output_type=rpc__pb2._RESPONSELISTWALLET,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='remove_wallet',
    full_name='forge_abi.WalletRpc.remove_wallet',
    index=4,
    containing_service=None,
    input_type=rpc__pb2._REQUESTREMOVEWALLET,
    output_type=rpc__pb2._RESPONSEREMOVEWALLET,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='declare_node',
    full_name='forge_abi.WalletRpc.declare_node',
    index=5,
    containing_service=None,
    input_type=rpc__pb2._REQUESTDECLARENODE,
    output_type=rpc__pb2._RESPONSEDECLARENODE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_WALLETRPC)

DESCRIPTOR.services_by_name['WalletRpc'] = _WALLETRPC


_STATISTICRPC = _descriptor.ServiceDescriptor(
  name='StatisticRpc',
  full_name='forge_abi.StatisticRpc',
  file=DESCRIPTOR,
  index=5,
  serialized_options=None,
  serialized_start=2464,
  serialized_end=3262,
  methods=[
  _descriptor.MethodDescriptor(
    name='get_forge_statistics',
    full_name='forge_abi.StatisticRpc.get_forge_statistics',
    index=0,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETFORGESTATISTICS,
    output_type=rpc__pb2._RESPONSEGETFORGESTATISTICS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_transactions',
    full_name='forge_abi.StatisticRpc.list_transactions',
    index=1,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLISTTRANSACTIONS,
    output_type=rpc__pb2._RESPONSELISTTRANSACTIONS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_assets',
    full_name='forge_abi.StatisticRpc.get_assets',
    index=2,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETASSETS,
    output_type=rpc__pb2._RESPONSEGETASSETS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_stakes',
    full_name='forge_abi.StatisticRpc.get_stakes',
    index=3,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETSTAKES,
    output_type=rpc__pb2._RESPONSEGETSTAKES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_top_accounts',
    full_name='forge_abi.StatisticRpc.get_top_accounts',
    index=4,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETTOPACCOUNTS,
    output_type=rpc__pb2._RESPONSEGETTOPACCOUNTS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_asset_transactions',
    full_name='forge_abi.StatisticRpc.list_asset_transactions',
    index=5,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLISTASSETTRANSACTIONS,
    output_type=rpc__pb2._RESPONSELISTASSETTRANSACTIONS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_blocks',
    full_name='forge_abi.StatisticRpc.list_blocks',
    index=6,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLISTBLOCKS,
    output_type=rpc__pb2._RESPONSELISTBLOCKS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_assets',
    full_name='forge_abi.StatisticRpc.list_assets',
    index=7,
    containing_service=None,
    input_type=rpc__pb2._REQUESTLISTASSETS,
    output_type=rpc__pb2._RESPONSELISTASSETS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_health_status',
    full_name='forge_abi.StatisticRpc.get_health_status',
    index=8,
    containing_service=None,
    input_type=rpc__pb2._REQUESTGETHEALTHSTATUS,
    output_type=rpc__pb2._RESPONSEGETHEALTHSTATUS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STATISTICRPC)

DESCRIPTOR.services_by_name['StatisticRpc'] = _STATISTICRPC

# @@protoc_insertion_point(module_scope)
